//-----------------------------------------------------------------------------
// File: platform_win.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "platform_win.h"

MP_Platform_WINDOWS::MP_Platform_WINDOWS()
{
}

MP_Platform_WINDOWS::~MP_Platform_WINDOWS()
{
}

// eng: Returns time in milliseconds
// rus:    
unsigned long MP_Platform_WINDOWS::GetTick()
{
	return GetTickCount();
}

// eng: Returns path to folder with emitters
// rus:      ptc-
const char* MP_Platform_WINDOWS::GetPathToPTC()
{
	if (ptc_path.empty())
	{
		wchar_t path[MAX_PATH];

#ifndef TEST
		GetModuleFileNameW(NULL, path, MAX_PATH); 
		int len=wcslen(path);
		for (int i=len-1;i>=0;i--)
		{
			if (path[i]=='\\')
			{
				path[i]=0;
				break;
			}
		}
#else
		GetCurrentDirectoryW(MAX_PATH, path);
#endif

		ptc_path=wchar_to_utf8(path);
		ptc_path+='\\';
	}

	return ptc_path.c_str();
}

// eng: Returns path to folder with textures
// rus:      
const char* MP_Platform_WINDOWS::GetPathToTexture()
{
	if (texture_path.empty())
	{
		texture_path=GetPathToPTC();
		texture_path+="textures\\";
	}

	return texture_path.c_str();
}

// eng: Returns path to folder which could be used by wrapper to store temporary files. If NULL is returned the temporary files are saved in RAM.
// rus:     .   NULL,       .
const char* MP_Platform_WINDOWS::GetPathToTemp()
{
	if (temp_path.empty())
	{
		wchar_t path[MAX_PATH];
		GetTempPathW(MAX_PATH, path);
		temp_path=wchar_to_utf8(path);
	}

	return temp_path.c_str();
}

// eng: Finds the first ptc-file in emitters folder
// rus:   ptc-    
const char* MP_Platform_WINDOWS::GetFirstFile()
{
	std::string ptc_path=GetPathToPTC();
	ptc_path+="*.ptc";
	const wchar_t* mask=utf8_to_wchar(ptc_path.c_str());
	hFindFile=FindFirstFileW(mask,&fd);
	if (hFindFile!=INVALID_HANDLE_VALUE)
	{
		file=wchar_to_utf8(fd.cFileName);
		return file.c_str();
	}
	FindClose(hFindFile);
	return NULL;
}

// eng: Finds the next ptc-file in emitters folder
// rus:   ptc-    
const char* MP_Platform_WINDOWS::GetNextFile()
{
	if (FindNextFileW(hFindFile,&fd))
	{
		file=wchar_to_utf8(fd.cFileName);
		return file.c_str();
	}
	FindClose(hFindFile);
	return NULL;
}

// eng: Converts wchar_t string into utf8
// rus:    wchar_t    utf8
const char* MP_Platform_WINDOWS::wchar_to_utf8(const wchar_t* str)
{
	const char* str8=(const char*)Magic_UTF16to8((const unsigned short*)str);
	return str8;
}

// eng: Converts utf8 string into wchar_t
// rus:    utf8    wchar_t
const wchar_t* MP_Platform_WINDOWS::utf8_to_wchar(const char* str)
{
	const short* str16=(const short*)Magic_UTF8to16((const unsigned char*)str);
	return (const wchar_t*)str16;
}

// ru:   ( "remove")
// en: Deleting of file (analogue "remove")
int MP_Platform_WINDOWS::RemoveFile(const char* file)
{
	return _wremove(utf8_to_wchar(file));
}