#include "mp_wrap.h"

#include <pf/file.h> 
#include <pf/assetmanager.h>

#ifdef MAGIC_3D
#include <pf/3deffect.h>
#else
#include <pf/2deffect.h>
#endif

#pragma warning ( disable : 4996)

PlaygroundBuffer buffer;

// eng: Loading of texture from memory
// rus:    
TTextureRef LoadImageMemory(const char* file, const char* mode, void* data, int length)
{
	std::string temp_folder="magic_temp";
	std::string path="common:";
	path+=temp_folder;
	path+="/";
	path+=file;
	std::string path_param=path;
	if (mode)
	{
		std::string mode_data=mode;
		if (!mode_data.empty())
		{
			path_param+='?';
			path_param+=mode_data;
		}
	}

	TFile::AddMemoryFile(path.c_str(), (void*)data, length);

	TTextureRef texture=TTexture::Get(path_param.c_str());
	texture->LoadNow();

	TFile::DeleteFile(path.c_str());

	return texture;
}

// eng: Loading of textures from file
// rus:    
TTextureRef LoadImage(const char* file, const char* mode)
{
	TTextureRef texture;
	MP_Manager& MP=MP_Manager::GetInstance();

	std::string texture_file=MP.GetPathToTexture();
	texture_file+=file;
	wchar_t fmode[3];
	fmode[0]='r';
	fmode[1]='b';
	fmode[2]=0;
	const wchar_t* texture_file_utf16=(const wchar_t*)Magic_UTF8to16((const unsigned char*)texture_file.c_str());
	FILE* stream=_wfopen(texture_file_utf16, fmode);

	if (stream)
	{
		fseek(stream,0,SEEK_END);
		long len=ftell(stream);
		fseek(stream,0,SEEK_SET);

		char* data=new char[len];
		fread(data,1,len,stream);
		fclose(stream);
		texture=LoadImageMemory(file, mode, (void*)data, len);
		delete []data;
	}

	return texture;
}

void TMat4ToMagicMatrix(TMat4* mat4, MAGIC_MATRIX* m)
{
	for (int i=0;i<4;i++)
	{
		for (int j=0;j<4;j++)
			m->m[i][j]=(*mat4)[i][j];
	}
}

void MagicMatrixToTMat4(MAGIC_MATRIX* m, TMat4* mat4)
{
	for (int i=0;i<4;i++)
	{
		for (int j=0;j<4;j++)
			(*mat4)[i][j]=m->m[i][j];
	}
}

// eng: Class storing the textural atlas
// rus: ,    
MP_Atlas_WRAP::MP_Atlas_WRAP(int width, int height, const char* file) : MP_Atlas(width, height, file)
{
	if (file_name.empty())
	{
		texture=TTexture::Create(width,height,true);
		texture->Clear();
	}
	else
	{
		texture=LoadImage(file_name.c_str(), "alpha,norotate");
	}
}

void MP_Atlas_WRAP::Destroy()
{
	texture.reset();
}

// eng: Loading of frame texture
// rus:   
void MP_Atlas_WRAP::LoadTexture(const MAGIC_CHANGE_ATLAS* c)
{
	std::string mode="?alpha,slow";

	TTextureRef texture_frame;

	if (c->data)
	{
		// rus:     
		texture_frame=LoadImageMemory(c->file, mode.c_str(), (void*)c->data, c->length);
	}
	else
	{
		// rus:     
		texture_frame=LoadImage(c->file, mode.c_str());
	}

	int frame_width=c->width;
	int frame_height=c->height;
	float scale_x=((float)frame_width)/((float)texture_frame->GetWidth());
	float scale_y=((float)frame_height)/((float)texture_frame->GetHeight());

	TColor32* from;
	uint32_t pitch_from;
	texture_frame->Lock(&from, &pitch_from, TTexture::kLockReadOnly);

	TColor32* to;
	uint32_t pitch_to;
	texture->Lock(&to,&pitch_to,TTexture::kLockWriteOnly);

	for (int i=0;i<frame_width;i++)
	{
		for (int j=0;j<frame_height;j++)
		{
			int i2=(int)(((float)i)/scale_x);
			int j2=(int)(((float)j)/scale_y);

			TColor32 color=from[j2*pitch_from+i2];
			to[(c->y+j)*pitch_to+i+(c->x)]=color;
		}
	}

	texture->Unlock();
	texture_frame->Unlock();
}

// eng: Cleaning up of rectangle
// rus:  
void MP_Atlas_WRAP::CleanRectangle(const MAGIC_CHANGE_ATLAS* c)
{
	
	TColor32* pixel;
	uint32_t pitch;
	texture->Lock(&pixel,&pitch,TTexture::kLockWriteOnly);

	for (int i=0;i<c->width;i++)
	{
		for (int j=0;j<c->height;j++)
		{
			int i2=(int)i+c->x;
			int j2=(int)j+c->y;

			pixel[j2*pitch+i2]=0;
		}
	}

	texture->Unlock();
}

// eng: Beginning of emitter drawing
// rus:   
void MP_Atlas_WRAP::Draw(MAGIC_PARTICLE_VERTEXES* vertexes)
{
	MP_Vert* vertex=buffer.NextParticles();
	TColor32 color=vertexes->color;

	vertex->pos.x=vertexes->vertex1.x;
	vertex->pos.y=vertexes->vertex1.y;
	#ifdef MAGIC_3D
	vertex->pos.z=vertexes->vertex1.z;
	#else
	vertex->pos.z=0.f;
	#endif
	vertex->uv.x=vertexes->u1;
	vertex->uv.y=vertexes->v1;
	vertex->color=color;
	vertex++;

	vertex->pos.x=vertexes->vertex2.x;
	vertex->pos.y=vertexes->vertex2.y;
	#ifdef MAGIC_3D
	vertex->pos.z=vertexes->vertex2.z;
	#else
	vertex->pos.z=0.f;
	#endif
	vertex->uv.x=vertexes->u2;
	vertex->uv.y=vertexes->v2;
	vertex->color=color;
	vertex++;

	vertex->pos.x=vertexes->vertex3.x;
	vertex->pos.y=vertexes->vertex3.y;
	#ifdef MAGIC_3D
	vertex->pos.z=vertexes->vertex3.z;
	#else
	vertex->pos.z=0.f;
	#endif
	vertex->uv.x=vertexes->u3;
	vertex->uv.y=vertexes->v3;
	vertex->color=color;
	vertex++;

	vertex->pos.x=vertexes->vertex4.x;
	vertex->pos.y=vertexes->vertex4.y;
	#ifdef MAGIC_3D
	vertex->pos.z=vertexes->vertex4.z;
	#else
	vertex->pos.z=0.f;
	#endif
	vertex->uv.x=vertexes->u4;
	vertex->uv.y=vertexes->v4;
	vertex->color=color;
};

// eng: Setting of intense
// rus:  
void MP_Atlas_WRAP::SetIntense(bool intense)
{
	#ifdef MAGIC_3D
	if(intense)
		device->effect3d->SetBlendMode(TEffect::kBlendAdditiveAlpha);
	else
		device->effect3d->SetBlendMode(TEffect::kBlendNormalAlphaAccumulate);
	#else
	TRenderer* renderer=TRenderer::GetInstance();
	TEffectRef effect=renderer->GetDefault2dEffect();
	if(intense)
		effect->SetBlendMode(TEffect::kBlendAdditiveAlpha);
	else
		effect->SetBlendMode(TEffect::kBlendNormalAlphaAccumulate);
	#endif
}

// eng: End of particles drawing
// rus:   
void MP_Atlas_WRAP::EndDrawParticles()
{
	int k_particles=buffer.GetCount();
	if (k_particles)
	{
		TVertexSet vertex_array(buffer.GetVertexArray(),k_particles*4);

		TRenderer* renderer=TRenderer::GetInstance();
		#ifdef MAGIC_3D
		device->effect3d->SetTexture(texture);
		renderer->SetEffect(device->effect3d);
		#else
		T2dEffectRef effect=renderer->GetDefault2dEffect();
		effect->SetTexture(texture);
		#endif

		renderer->DrawIndexedVertices(TRenderer::kDrawTriangles, vertex_array, buffer.GetIndexArray(), k_particles*6);

		buffer.Clear();
	}
}

// ------------------------------------------------------------------------------------

// eng: Class controlling drawing
// rus: ,   
MP_Device_WRAP::MP_Device_WRAP(int width, int height) : MP_Device(width,height)
{
	#ifdef MAGIC_3D
	effect3d=T3dEffect::Create();
	effect3d->DisableAmbientColor();

	effect3d->SetFilteringMode(TBasicEffect::kFilterLinear);
	effect3d->SetTextureMapMode(TBasicEffect::kMapClamp, TBasicEffect::kMapClamp);

	effect3d->SetZBufferWrite(false);
	effect3d->SetZBufferTest(false);

	effect3d->SetColorWrite(true);
	effect3d->SetAlphaWrite(false);

	effect3d->SetCullMode(TEffect::kCullNone);

	effect3d->SetNextEffect();

	#endif
}

// eng: Setting of scene
// rus:  
void MP_Device_WRAP::SetScene3d()
{
	#ifdef MAGIC_3D

	TRenderer* renderer=TRenderer::GetInstance();

	renderer->SetPerspectiveProjection(0.0f, 100.0f, (float)(MAGIC_PI * 0.15f), ((float)window_width)/((float)window_height)); 
  
	TVec3 eye(0.f,0.f,-840.f);
	TVec3 at(0.f,0.f,0.f);
	TVec3 up(0.f,1.f,0.f);
	renderer->SetView(eye, at, up);

	TMat4 view;
	renderer->GetViewMatrix(&view);
	MAGIC_MATRIX v;
	TMat4ToMagicMatrix(&view, &v);
	Magic_SetViewMatrix(&v);

	#endif
}

// eng: Beginning of scene drawing
// rus:   
void MP_Device_WRAP::BeginScene()
{
	TRenderer* renderer=TRenderer::GetInstance();

	#ifdef MAGIC_3D

	TMat4 world=TMat4::IDENTITY;
	renderer->SetWorldMatrix(world);

	TRect view_port;
	view_port.x1=0;
	view_port.y1=0;
	view_port.x2=window_width;
	view_port.y2=window_height;
	renderer->SetViewport(view_port);

	renderer->Begin3d();

	SetScene3d();
	
	#else
	renderer->Begin2d();
	#endif

	renderer->FillRect(TURect(0,0,window_width,window_height), TColor(0,0,0,0));
}

// eng: End of scene drawing
// rus:   
void MP_Device_WRAP::EndScene()
{
	TRenderer* renderer=TRenderer::GetInstance();
	#ifdef MAGIC_3D
	renderer->End3d();
	#else
	renderer->End2d();
	TEffectRef effect= renderer->GetDefault2dEffect();
	effect->SetBlendMode(TEffect::kBlendNormalAlphaAccumulate);
	#endif
}
