#ifndef MAGIC_WRAPPER
#define MAGIC_WRAPPER

#include "mp.h"
#include <pf/texture.h> 


#ifdef MAGIC_3D
#define MP_Vert TLitVert
#else
#define MP_Vert TTransformedLitVert
#endif

// eng: Class storing the textural atlas
// rus: ,    
class MP_Atlas_WRAP : public MP_Atlas
{
protected:
	TTextureRef texture;

public:
	MP_Atlas_WRAP(int width, int height, const char* file);

	// eng: Destroy atlas texture
	// rus:   
	virtual void Destroy();

	// eng: Loading of frame texture
	// rus:   
	virtual void LoadTexture(const MAGIC_CHANGE_ATLAS* c);

	// eng: Cleaning up of rectangle
	// rus:  
	virtual void CleanRectangle(const MAGIC_CHANGE_ATLAS* c);

	// eng: Beginning of emitter drawing
	// rus:   
	virtual void Draw(MAGIC_PARTICLE_VERTEXES* vertexes);

	// eng: End of particles drawing
	// rus:   
	virtual void EndDrawParticles();

	// eng: Setting of intense
	// rus:  
	virtual void SetIntense(bool intense);
};

// eng: Class controlling drawing
// rus: ,   
class MP_Device_WRAP : public MP_Device
{
public:
	#ifdef MAGIC_3D
	T3dEffectRef effect3d;
	#endif

public:
	MP_Device_WRAP(int width, int height);

	// eng: Creating
	// rus: 
	virtual bool Create(){return true;}

	// eng: Destroying
	// rus: 
	virtual void Destroy(){}

	// eng: Setting of scene
	// rus:  
	virtual void SetScene3d();

	// eng: Beginning of scene drawing
	// rus:   
	virtual void BeginScene();

	// eng: End of scene drawing
	// rus:   
	virtual void EndScene();

	// eng: Indicates that device is lost
	// rus:     
	virtual bool IsLost(){return false;}
};

// eng: Class of vertexes buffer
// rus:   
class PlaygroundBuffer
{
protected:
	int k_particles;
	int max_particles;

	MP_Vert* m_vertex;
	uint16_t* m_index;

public:
	PlaygroundBuffer()
	{
		k_particles=0;
		max_particles=10;

		m_vertex=(MP_Vert*)malloc(max_particles*sizeof(MP_Vert)*4);
		m_index=(uint16_t*)malloc(max_particles*sizeof(uint16_t)*6);

		for (int i=0;i<max_particles;i++)
		{
			uint16_t vertex=((uint16_t)i)*4;
			MP_Vert* vert=&(m_vertex[vertex]);
			for (int j=0;j<4;j++)
			{
				vert->pos.z=0.f;
				//vert->RESERVED=0;
				//vert->specular=0;
				vert++;
			}

			uint16_t* ind=&(m_index[i*6]);
			*ind=vertex;
			ind++;
			*ind=vertex+1;
			ind++;
			*ind=vertex+2;
			ind++;
			*ind=vertex+3;
			ind++;
			*ind=vertex;
			ind++;
			*ind=vertex+2;
			ind++;
		}
	}

	virtual ~PlaygroundBuffer()
	{
		free(m_vertex);
		m_vertex=NULL;
		free(m_index);
		m_index=NULL;
		k_particles=0;
		max_particles=0;
	}

	// rus:     
	int GetCount(){return k_particles;}

	// rus:   
	MP_Vert* GetVertexArray(){return m_vertex;}

	// rus:    
	uint16_t* GetIndexArray(){return m_index;}

	// rus:   
	void Clear(){k_particles=0;}

	// rus:     
	MP_Vert* NextParticles()
	{
		if (k_particles==max_particles)
		{
			// rus:   
			int size=sizeof(MP_Vert);
			int max_particles2=max_particles+max_particles/4;
			m_vertex=(MP_Vert*)realloc(m_vertex,size*(max_particles2)*4);

			size=sizeof(uint16_t);
			m_index=(uint16_t*)realloc(m_index,size*(max_particles2)*6);

			for (int i=max_particles;i<max_particles2;i++)
			{
				uint16_t vertex=((uint16_t)i)*4;
				MP_Vert* vert=&(m_vertex[vertex]);
				for (int j=0;j<4;j++)
				{
					vert->pos.z=0.f;
					//vert->RESERVED=0;
					//vert->specular=0;
					vert++;
				}

				uint16_t* ind=&(m_index[i*6]);
				*ind=vertex;
				ind++;
				*ind=vertex+1;
				ind++;
				*ind=vertex+2;
				ind++;
				*ind=vertex+3;
				ind++;
				*ind=vertex;
				ind++;
				*ind=vertex+2;
				ind++;
			}

			max_particles=max_particles2;
		}

		MP_Vert* vert=&(m_vertex[k_particles*4]);
		k_particles++;
		return vert;
	}
};


class TTaskRestoreTexture : public TTask
{
public:
	TTaskRestoreTexture() : TTask(){}

	bool DoTask()
	{
		MP_Manager::GetInstance().RestoreAtlas();
		return true;
	}
};

#endif