//------------------------------------------------------------------
// Copyright (c) 2004-2008 PlayFirst, Inc. All rights reserved.
// Part of the Playground SDK(tm).
//
// This file contains proprietary information of PlayFirst, Inc.
// It is provided subject to a license agreement. It may not be
// reproduced, used, disclosed, sold or transferred without the
// prior written consent of PlayFirst, Inc.
//------------------------------------------------------------------
#include "mp_wrap.h"
#include "platform_pg.h"

#include "game.h"
#include "globaldefines.h"
#include "settings.h"

#include <pf/pflib.h>
#include <pf/platform.h> 

#include "pf/debug.h"


PFTYPEIMPL_DC(TGame);

TGame::TGame()
{
	TGameState::GetInstance()->SetState("++PlayedSimpleGame");

	TPlatform::SetConfig( "vsync", "1" );

	int client_wi=SCREEN_WIDTH;
	int client_he=SCREEN_HEIGHT;

	device=new MP_Device_WRAP(client_wi, client_he);
	device->Create();

	MP_Manager& MP=MP_Manager::GetInstance();

	MP_Platform* platform=new MP_Platform_WINDOWS_PG;
	MP.Initialization(platform, MAGIC_INTERPOLATION_ENABLE, MAGIC_NOLOOP, MAGIC_CHANGE_EMITTER_DEFAULT, 1024, 1024, 1, 1.f, 0.1f, true);

	// eng: find of all ptc-files in folder
	// rus:   ptc-  
	MP.LoadAllEmitters();
	
	MP.RefreshAtlas();

	MP.CloseFiles();

	MP.Stop();

	cur=MP.GetFirstEmitter();

	MP_Emitter* emitter=MP.GetEmitter(cur);
	emitter->SetState(MAGIC_STATE_UPDATE);

	#ifndef MAGIC_3D
	// eng: locate emitters the same as editor
	// rus:   ,     
	HM_EMITTER hmEmitter=MP.GetFirstEmitter();
	while (hmEmitter)
	{
		Magic_CorrectEmitterPosition(hmEmitter, client_wi, client_he);
		hmEmitter=MP.GetNextEmitter(hmEmitter);
	}
	#endif

	restore_texture=new TTaskRestoreTexture();
	TPlatform::GetInstance()->AdoptTextureRefreshListener(restore_texture);
}

TGame::~TGame()
{
	TGameState::GetInstance()->SetState("begin-status","SimpleGameComplete");
	TGameState::GetInstance()->SetState("add-state","LastGameHiScore");
	TGameState::GetInstance()->SetState("end-status","SimpleGameComplete");

	ScriptUnregisterFunction(TWindowManager::GetInstance()->GetScript(),"SendGameMessage");

	TPlatform::GetInstance()->OrphanTextureRefreshListener(restore_texture);

	if (restore_texture)
	{
		delete restore_texture;
		restore_texture=NULL;
	}

	// eng: Delete particle manager
	// rus:   
	MP_Manager& MP=MP_Manager::GetInstance();
	MP.Destroy();

	if (device)
	{
		device->Destroy();
		delete device;
		device=NULL;
	}
}

void TGame::Draw()
{
	MP_Manager& MP=MP_Manager::GetInstance();
	MP.Render();
}

/**
 * Called if you have initiated a window animation with
 * TWindow::StartWindowAnimation.
 *
 * @return True to continue animating. False to stop.
 */
bool TGame::OnTaskAnimate()
{
    // And don't forget to update the screen
	TWindowManager::GetInstance()->InvalidateScreen();

	if (TGameState::GetInstance()->QueryJumpToMenu())
	{
		SendGameMessage("quit");
	}
	if (TGameState::GetInstance()->QueryRestartMode())
	{
		// Restart our game mode!
	}

	MP_Manager& MP=MP_Manager::GetInstance();

	MP.UpdateByTimer();

	MP_Emitter* emitter=MP.GetEmitter(cur);
	if (emitter->GetState()==MAGIC_STATE_STOP)
	{
		cur=MP.GetNextEmitter(cur);
		if (!cur)
			cur=MP.GetFirstEmitter();

		emitter=MP.GetEmitter(cur);
		emitter->SetState(MAGIC_STATE_UPDATE);
	}
	
    // Keep animating
	return true;
}

/**
 * Initialize the Window. Called by the system only in
 * Lua initialization.
 *
 * When you create your own custom window, this is where you put
 * your own custom initialization that needs to happen before
 * children are created. Fundamental window initialization is
 * handled in every class by this function, so <b>when you
 * override this function you almost always want to call your
 * base class to handle base class initialization.</b>
 */
void TGame::Init(TWindowStyle& style)
{
	TScript * s = TWindowManager::GetInstance()->GetScript();
	ScriptRegisterMemberDirect(s,"SendGameMessage",this,TGame::SendGameMessage);

    // Start the window animation
    StartWindowAnimation(30);

	// Send keyboard events our way if no one else uses them
	FindParentModal()->SetDefaultFocus(this);

	pfparent::Init(style);
}

/**
 * Mouse down handler.
 *
 * @param point      Location of mouse press in client coordinates.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TGame::OnMouseDown(const TPoint & /*point*/)
{
    return true;
}


void TGame::SendGameMessage( str message )
{
	if (message=="quit")
	{
		// Call the helper function in TSettings that pops all of the windows off the stack.
		// It assumes that you still have the forever loop in mainloop.lua that recreates the
		// main menu screen when it's been popped.
		TSettings::GetInstance()->QuitToMainMenu();
	}
}

/**
 * Raw key hit on keyboard.
 *
 * @param key    Key pressed on keyboard.
 * @param flags  TEvent::EKeyFlags mask representing the state of other keys on the keyboard
 *               when this key was hit.
 *
 * @return true if message was handled, false to keep searching
 *         for a handler.
 */
bool TGame::OnKeyDown(char key, uint32_t /*flags*/)
{
	if (key == TEvent::kEscape)
	{
		TWindowManager::GetInstance()->DoLuaString(this,"DoModal('scripts/pause.lua')");
		return true;
	}

	return false;
}
