//------------------------------------------------------------------
// Copyright (c) 2004-2009 PlayFirst, Inc. All rights reserved.
//
// This material is the confidential trade secret and proprietary
// information of PlayFirst, Inc. It may not be reproduced, used, disclosed,
// sold or transferred without the prior written consent of PlayFirst, Inc.
// Any use is subject to license by PlayFirst, Inc.
//------------------------------------------------------------------


/**
 * @file
 * Implementation for class TLoading
 */


#include "loading.h"
#include "pf/debug.h"
#include "pf/windowmanager.h"
#include "pf/windowstyle.h"
#include "pf/luatable.h"
#include "pf/renderer.h"
#include "pf/script.h"

PFTYPEIMPL_DC(TLoading);

TLoading::TLoading() :
	mDone(false)
{
}

TLoading::~TLoading()
{
	VERBOSE_TRACE();
}

/**
 * Called if you have initiated a window animation with
 * TWindow::StartWindowAnimation.
 *
 * @return True to continue animating. False to stop.
 */
bool TLoading::OnTaskAnimate()
{
	TWindowManager::GetInstance()->InvalidateScreen();
	return true;
}

/**
 * Initialize the Window. Called by the system only in
 * Lua initialization.
 *
 * When you create your own custom window, this is where you put
 * your own custom initialization that needs to happen before
 * children are created. Fundamental window initialization is
 * handled in every class by this function, so <b>when you
 * override this function you almost always want to call your
 * base class to handle base class initialization.</b>
 *
 * @param style  The Lua style that was in effect when this window was
 *               created. This style contains all parameters specified
 *               explicitly for the window as well as parameters defined
 *               in the current style. Parameters set locally override
 *               ones in the style.
 */
void TLoading::Init( TWindowStyle & style )
{
	StartWindowAnimation(10);

	TLuaTable * bar = style.GetTable("loadingbar");

	mLoadingBarRect.x1 = (int32_t)bar->GetNumber("x");
	mLoadingBarRect.y1 = (int32_t)bar->GetNumber("y");
	mLoadingBarRect.x2 = (int32_t)bar->GetNumber("w");
	mLoadingBarRect.y2 = (int32_t)bar->GetNumber("h");

	delete bar; bar = NULL;

	TWindow::Init(style);

	FixSize( &mLoadingBarRect.x1,&mLoadingBarRect.x2,&TWindow::GetWindowWidth );
	FixSize( &mLoadingBarRect.y1,&mLoadingBarRect.y2,&TWindow::GetWindowHeight );

	mLoadingBarRect.GetBottomRight()+= mLoadingBarRect.GetTopLeft();

}

#define INTERP(p,A,B) ((((B)-(A))*p)+(A))

/**
 * Draw the window. Derived classes will override this
 * function and provide the draw functionality.
 */
void TLoading::Draw()
{
	TRect drawRect = mLoadingBarRect;
	TReal amount = CheckLoaded();
	drawRect.x2 = (int32_t) INTERP(	amount,drawRect.x1+1,drawRect.x2 );

	TRenderer::GetInstance()->FillRect( drawRect, TColor(1,0,0,1) );
}


TReal TLoading::CheckLoaded()
{
	TReal amount = TWindowManager::GetInstance()->GetGUIAssetsAmountLoaded();
	if (amount == 1.0)
	{
		Done();
	}
	return amount;
}

void TLoading::Done()
{
	if (!mDone)
	{
		VERBOSE_TRACE();
		TWindowManager::GetInstance()->GetScript()->DoLuaString(
			str::getFormatted("gCommandQueue[ #gCommandQueue+1 ]=function() CloseWindow() ; DebugOut('foo'); end;") );
	}

	mDone = true;
}

/**
 * Handle any initialization or setup that is required when
 * this window is assigned to a new parent. No initialization
 * of the window has happened prior to this call.
 *
 * @return True on success; false on failure.
 *
 * @see Init
 * @see PostChildrenInit
 */
bool TLoading::OnNewParent()
{
	// If the loading bar is almost done, just drop out immediately
	if (CheckLoaded()>0.75F)
	{
		GetParent()->SetVisible(false);
		Done();
		return false;
	}
	return true;
}
