//------------------------------------------------------------------
// Copyright (c) 2004-2008 PlayFirst, Inc. All rights reserved.
// Part of the Playground SDK(tm).
//
// This file contains proprietary information of PlayFirst, Inc.
// It is provided subject to a license agreement. It may not be
// reproduced, used, disclosed, sold or transferred without the
// prior written consent of PlayFirst, Inc.
//------------------------------------------------------------------



#define PLAYFIRST_MAIN

#include "main.h"
#include "globaldefines.h"
#include "game.h"
#include "mainmenu.h"
#include "loading.h"
#include "options.h"
#include "settings.h"
#include "hiscore.h"
#include "serversubmit.h"
#include "chooseplayer.h"
#include "customtextedit.h"
#include "credits.h"
#include "key.h"
#include "rolloverwindow.h"

#include <pf/windowmanager.h>
#include <pf/image.h>
#include <pf/button.h>
#include <pf/texture.h>
#include <pf/event.h>
#include <pf/task.h>
#include <pf/stringtable.h>
#include <pf/screen.h>
#include <pf/slider.h>
#include <pf/script.h>
#include <pf/gamestate.h>

#include <pf/prefsdb.h>

/*
 * Early initialization of Playground values. TPlatform has not yet
 * been initialized here, so only static functions on TPlatform are
 * legal! At present this is basically only SetConfig(). This function
 * is really designed only for setting up values that TPlatform will need
 * during initialization; most application initialization still belongs in
 * Main().
 */
void pfPreInit()
{
	// Set the name of the game. This name will be used in the user: and
	// common: data folder names.
	TPlatform::SetConfig( TPlatform::kGameName, "Playground Skeleton Application" );

	//TPlatform::SetConfig( TPlatform::kVerboseDebug, "1" );

	// Set the PlayFirst game information. All PlayFirst games MUST set the
	// information below in order to successfully communicate with PlayFirst
	// services, such as the hiscore system. Removing this code will cause
	// the hiscore system to not work.
	// DO NOT REMOVE THIS CODE IF YOU PLAN ON USING PLAYFIRST SERVICES
	// ---------------------------------------------------------------
	TPlatform::SetConfig(TPlatform::kEncryptionKey,ENCRYPTION_KEY);

	// Set the hiscore handle of the Playground game.
	TPlatform::SetConfig(TPlatform::kPFGameHandle,PFGAMEHANDLE);

	// Publisher defaults to PlayFirst; you can change it here
	//TPlatform::SetConfig(TPlatform::kPublisherName,"PlayFirst");

	for (int i = 0; i < sizeof(PFGAMEMODENAMES)/sizeof(char *); i++)
	{
		str name = str::getFormatted("%s%d", TPlatform::kPFGameModeName, i+1);
		TPlatform::SetConfig(name,PFGAMEMODENAMES[i]);
	}

	for (int i = 0; i < sizeof(PFGAMEMEDALNAMES)/sizeof(char *); i++)
	{
		str name = str::getFormatted("%s%d", TPlatform::kPFGameMedalName, i+1);
		TPlatform::SetConfig(name,PFGAMEMEDALNAMES[i]);
	}
	// ---------------------------------------------------------------
}

/*
 * This function sets up the metrics-gathering code in Playground SDK.
 * If you're attempting to understand the basics of Playground right
 * now, you can safely ignore it until later. :)
 *
 * See the documentation on TGameState and metrics for more
 * information.
 */
static void InitializeGameState()
{
	TGameState * gs = TGameState::GetInstance();

	// Start the session timer
	gs->SetState("SessionTime",1);

    // Create a metric line that will be written when the
    // session completes.
	gs->SetState("begin-session","SessionData");
	gs->SetState("add-value", TPlatform::GetConfig(TPlatform::kGameVersion)); // Write our version
	gs->SetState("add-timer","SessionTime");		// Write the session timer
	gs->SetState("add-state","LastGameHiScore");	// Write the last game's high score
	gs->SetState("end-session","SessionData");

    // Create a metric line that will be written when the
    // user has played the game for 15 minutes cumulatively
	gs->SetState("begin-elapsed(15)","Progress-15");
	gs->SetState("add-state","PlayedSimpleGame");	// Write how many times they played the Simple Game
	gs->SetState("end-elapsed(15)","Progress-15");

    // Create a metric line that will be written when the
    // user has played the game for 30 minutes cumulatively
	gs->SetState("begin-elapsed(30)","Progress-30");
	gs->SetState("add-state","PlayedSimpleGame");	// Write how many times they played the Simple Game
	gs->SetState("end-elapsed(30)","Progress-30");

    // Create a metric line that will be written when the
    // user has played the game for 45 minutes cumulatively
	gs->SetState("begin-elapsed(45)","Progress-45");
	gs->SetState("add-state","PlayedSimpleGame");	// Write how many times they played the Simple Game
	gs->SetState("end-elapsed(45)","Progress-45");

    // Create a metric line that will be written when the
    // user has played the game for 59 minutes cumulatively
	gs->SetState("begin-elapsed(59)","Progress-59");
	gs->SetState("add-state","PlayedSimpleGame");	// Write how many times they played the Simple Game
	gs->SetState("end-elapsed(59)","Progress-59");
}

/**
 * This function is called by Playground after basic engine
 * initialization, so you can set up real values here.
 *
 * Return value should be true if you want the game to continue
 * initializing. Otherwise return false and Playground should
 * exit.
 *
 * The string table has been loaded by this point.
 */
bool pfInit( const char * /*cmdLine*/ )
{
	#ifdef MAGIC_3D
	TPlatform::SetConfig( TPlatform::kWindowTitle, "Magic Particles 3D");
	#else
	TPlatform::SetConfig( TPlatform::kWindowTitle, "Magic Particles");
	#endif

	InitializeGameState();

	TSettings::CreateSettings();
	TSettings::GetInstance()->InitGameToSettings();

	TWindowManager * wm = TWindowManager::GetInstance();
	wm->AddWindowType("GameWindow",TGame::ClassId());
	wm->AddWindowType("MainMenu",TMainMenu::ClassId());
	wm->AddWindowType("Loading",TLoading::ClassId());
	wm->AddWindowType("OptionsWindow",TOptions::ClassId());
	wm->AddWindowType("HiscoreWindow",THiscore::ClassId());
	wm->AddWindowType("ChoosePlayerWindow",TChoosePlayer::ClassId());
	wm->AddWindowType("CreditsWindow",TCredits::ClassId());
	wm->AddWindowType("RolloverWindow", TRolloverWindow::ClassId());
	TServerSubmit::Register();

	// Overriding the default TTextEdit window.
	wm->AddWindowType("TextEdit",TCustomTextEdit::ClassId());

	// Start the Lua GUI script; this script will never exit
	// in a typical Playground application.
	wm->GetScript()->RunScript("scripts/mainloop.lua");

	return true;
}

bool pfEvent( const TEvent & event )
{
	if ( event.mType == TEvent::kClose )
	{
		TWindowManager::GetInstance()->GetScript()->RunScript("scripts/quitverify.lua");
	}

	if ( event.mType == TEvent::kFullScreenToggle )
	{
		TSettings::GetInstance()->UpdateFullScreen();
	}

	// Pass the event to the Window manager for further processing
	TWindowManager::GetInstance()->HandleEvent(&event);

	return true; // True to keep on going!
}

// You can put any periodic update events here. Or you can just register
// your events with TPlatform::AdoptTask() or TModalWindow::AdoptTask(),
// depending on whether the task should ALWAYS execute, or only when the
// modal window is on top.
void pfUpdate( float elapsedTime )
{
	TWindowManager::GetInstance()->Update(elapsedTime);
}

// Add any custom drawing code here you want. This function is called once
// per frame when it's time to draw.
//
// Most people won't need to modify this function at all. If you do want to add
// your own drawing code, don't forget you'll need to add a TRenderer::BeginDraw/EndDraw()
// pair. In TWindow::Draw()-derived functions, BeginDraw() has already been called for
// you.
void pfDraw()
{
	TWindowManager::GetInstance()->DrawStack();
}

// Add any of your own shut-down code here.
void pfShutDown()
{
	TSettings::DeleteSettings();
}

