//------------------------------------------------------------------
// Copyright (c) 2004-2008 PlayFirst, Inc. All rights reserved.
// Part of the Playground SDK(tm).
//
// This file contains proprietary information of PlayFirst, Inc.
// It is provided subject to a license agreement. It may not be
// reproduced, used, disclosed, sold or transferred without the
// prior written consent of PlayFirst, Inc.
//------------------------------------------------------------------


#include "options.h"
#include "settings.h"
#include "globaldefines.h"

#include <pf/slider.h>
#include <pf/windowmanager.h>
#include <pf/script.h>
#include <pf/message.h>
#include <pf/soundmanager.h>

PFTYPEIMPL_DC(TOptions);

TOptions::TOptions()
{
	TWindowManager * wm = TWindowManager::GetInstance();

	ScriptRegisterMemberDirect( wm->GetScript(), "SaveVolumes", this, TOptions::SaveVolumes );
}

TOptions::~TOptions()
{

}

void TOptions::PostChildrenInit(TWindowStyle & style)
{
	pfparent::PostChildrenInit(style);
	TWindow *window = GetChildWindow("fullscreen", -1);
	if (window)
	{
		TButton *button = window->GetCast<TButton>();
		if (button)
		{
			if (TSettings::GetInstance()->IsFullScreen())
			{
				button->SetOn(true);
			}
			else
			{
				button->SetOn(false);
			}
		}
	}

	window = GetChildWindow("mutebox", -1);
	if (window)
	{
		TButton *button = window->GetCast<TButton>();
		if (button)
		{
			int soundoff = TSettings::GetInstance()->GetPreferences()->GetInt("soundoff", 0, TPrefsDB::kCurrentUser);
			if (soundoff == 1)
			{
				button->SetOn(true);
			}
			else
			{
				button->SetOn(false);
			}
		}
	}

	window = GetChildWindow("sfxlevelslider", -1);
	if (window)
	{
		TSlider *slider = window->GetCast<TSlider>();
		if (slider)
		{
			int soundVal = TSettings::GetInstance()->GetPreferences()->GetInt("soundlevel", DEFAULT_SOUND_VOLUME, TPrefsDB::kCurrentUser);
			slider->SetValue((float)soundVal/1000.0f);
		}
	}

	window = GetChildWindow("musiclevelslider", -1);
	if (window)
	{
		TSlider *slider = window->GetCast<TSlider>();
		if (slider)
		{
			int soundVal = TSettings::GetInstance()->GetPreferences()->GetInt("musiclevel", DEFAULT_SOUND_VOLUME, TPrefsDB::kCurrentUser);
			slider->SetValue((float)soundVal/1000.0f);
		}
	}
}

void TOptions::SaveVolumes()
{
	TWindow *window;


	window = GetChildWindow("sfxlevelslider", -1);
	if (window)
	{
		TSlider *slider = window->GetCast<TSlider>();
		if (slider)
		{
			float sliderVal = slider->GetValue();
			TSettings::GetInstance()->GetPreferences()->SetInt("soundlevel", (int)(sliderVal * 1000.0f), TPrefsDB::kCurrentUser);
		}
	}

	window = GetChildWindow("musiclevelslider", -1);
	if (window)
	{
		TSlider *slider = window->GetCast<TSlider>();
		if (slider)
		{
			float sliderVal = slider->GetValue();
			TSettings::GetInstance()->GetPreferences()->SetInt("musiclevel", (int)(sliderVal * 1000.0f), TPrefsDB::kCurrentUser);
		}
	}
}


bool TOptions::OnMessage(TMessage * message)
{

	bool handled = false;
	if (message->mType == TMessage::kSliderValChanged)
	{
		if (message->mName == "sfxlevelslider")
		{
			TWindow *window = GetChildWindow("sfxlevelslider", -1);
			if (window)
			{
				TSlider *slider = window->GetCast<TSlider>();
				if (slider)
				{
					float val = slider->GetValue();
					TSoundManager::GetInstance()->SetTypeVolume(TSound::kSFXSoundGroup, val);
					handled = true;
				}

			}
		}
		else if (message->mName == "musiclevelslider")
		{
			TWindow *window = GetChildWindow("musiclevelslider", -1);
			if (window)
			{
				TSlider *slider = window->GetCast<TSlider>();
				if (slider)
				{
					float val = slider->GetValue();
					TSoundManager::GetInstance()->SetTypeVolume(TSound::kMusicSoundGroup, val);
					handled = true;
				}
			}
		}
	}


	return handled;
}
